{
Client Access Objects 1.5

caoDatabaseCatalog component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The cwbDatabaseCatalog automation object is used to retrieve information about
database objects (libraries, files, members) on an AS/400 system.  For example,
you can retrieve a list of libraries on the AS/400 or the list of files in a
library.  You can also retrieve information about SQL packages.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoDatabaseCatalog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  caoErrorMessage, caoDatabaseConnection, caoDatabaseDataFormat,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoDatabaseCatalog exception object

  EcaoDatabaseCatalog = class(Exception);

  TcaoCatDataRetrievalMode = (
    caodbSynchronousRetrieval,
    caodbAsynchronousRetrieval);

  TcaoCatDataToRetrieve = (
    caodbRetrieveData,
    caodbRetrieveDataFormat);

  TcaoDefaultFileAttributes = (
    caodbAllFilesAttributes,
    caodbPhysicalFilesAttributes,
    caodbLogicalFilesAttributes,
    caodbODBCTablesAttributes,
    caodbODBCViewsAttributes);

  TcaoDefaultFileInfoOrdering = (
    caodbDefaultCatalogOrdering,
    caodbODBCTableOrdering,
    caodbODBCTablePrivilegeOrder);

  TcaoDefaultNullableIndicator = (
    caodbNotNullable,
    caodbNullable);

  TcaoDefaultStatementType = (
    caodbAllStatements,
    caodbDeclareStatements,
    caodbSelectStatements,
    caodbExecStatements);

  TcaoCatDataToRetrieveSet = set of TcaoCatDataToRetrieve;

  TcaoDatabaseCatalog = class(TComponent)
  private
    { Private declarations }
    FReturnCode: Integer;
    FSetupCalled: Boolean;
    FdataRetrievalMode: TcaoCatDataRetrievalMode;
    FdataToRetrieve: TcaoCatDataToRetrieveSet;
    FdefaultFileAttributes: TcaoDefaultFileAttributes;
    FdefaultFileInfoOrdering: TcaoDefaultFileInfoOrdering;
    FdefaultNullableIndicator: TcaoDefaultNullableIndicator;
    FdefaultStatementType: TcaoDefaultStatementType;
    FcwbDatabaseCatalog: cwbDatabaseCatalog;
    FcaoErrorMessage: TcaoErrorMessage;
    FcaoDatabaseConnection: TcaoDatabaseConnection;
    FcaoDatabaseDataFormat: TcaoDatabaseDataFormat;
    function GetDataRetrievalMode: TcaoCatDataRetrievalMode;
    procedure SetDataRetrievalMode(RetMode: TcaoCatDataRetrievalMode);
    function GetDataToRetrieve: TcaoCatDataToRetrieveSet;
    procedure SetDataToRetrieve(DataRet: TcaoCatDataToRetrieveSet);
    function GetDefaultFileAttributes: TcaoDefaultFileAttributes;
    procedure SetDefaultFileAttributes(FileAtt: TcaoDefaultFileAttributes);
    function GetDefaultFileInfoOrdering: TcaoDefaultFileInfoOrdering;
    procedure SetDefaultFileInfoOrdering(FileOrd: TcaoDefaultFileInfoOrdering);
    function GetDefaultNullableIndicator: TcaoDefaultNullableIndicator;
    procedure SetDefaultNullableIndicator(NullInd: TcaoDefaultNullableIndicator);
    function GetDefaultStatementType: TcaoDefaultStatementType;
    procedure SetDefaultStatementType(StmtType: TcaoDefaultStatementType);
    function GetResultDataLength: Integer;
    function GetResultIndicatorCount: Integer;
  protected
    { Protected declarations }
    function ResultCodeText: String; virtual;
    function GetErrMsg: String; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Setup; virtual;
    procedure GetResultData(var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer); virtual;
    procedure GetResultDataFormat; virtual;
    procedure RetrieveLibraryInfo(const libraryName: WideString; retrieveInformation: Integer); virtual;
    procedure GetHostErrInfo(var errorClass: Smallint; var errorCode: Integer; var messageID: WideString; var message1: WideString; var message2: WideString); virtual;
    procedure RetrieveFieldInfo(const libraryName, fileName, fieldName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrieveFileInfo(const libraryName, fileName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrieveForeignKeyInfo(const foreignKeyLibraryName, foreignKeyFileName, primaryKeyLibraryName, primaryKeyFileName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrieveIndexInfo(const libraryName, fileName: WideString; indexType: Smallint; retrieveInformation: Integer); virtual;
    procedure RetrieveMemberInfo(const libraryName, fileName, memberName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrievePackageStmtInfo(const libraryName, packageName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrievePrimaryKeyInfo(const primaryKeyLibraryName, primaryKeyFileName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrieveRDBInfo(const RDBName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrieveRecordFormatInfo(const libraryName, fileName, formatName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrieveSpecialColumnInfo(const libraryName, fileName: WideString; retrieveInformation: Integer); virtual;
    procedure RetrieveSQLPackageInfo(const libraryName, packageName: WideString; retrieveInformation: Integer); virtual;
    procedure ChangeConnection(connection: IDispatch); virtual;
  published
    { Published declarations }
    property ReturnCode: Integer read FReturnCode default cwbOK;
    property dataRetrievalMode: TcaoCatDataRetrievalMode read GetDataRetrievalMode write SetDataRetrievalMode default caodbSynchronousRetrieval;
    property dataToRetrieve: TcaoCatDataToRetrieveSet read GetDataToRetrieve write SetDataToRetrieve default [caodbRetrieveData,caodbRetrieveDataFormat];
    property defaultFileAttributes: TcaoDefaultFileAttributes read GetDefaultFileAttributes write SetDefaultFileAttributes default caodbAllFilesAttributes;
    property defaultFileInfoOrdering: TcaoDefaultFileInfoOrdering read GetDefaultFileInfoOrdering write SetDefaultFileInfoOrdering default caodbDefaultCatalogOrdering;
    property defaultNullableIndicator: TcaoDefaultNullableIndicator read GetDefaultNullableIndicator write SetDefaultNullableIndicator default caodbNotNullable;    property defaultStatementType: TcaoDefaultStatementType read GetDefaultStatementType write SetDefaultStatementType default caodbAllStatements;
    property resultDataLength: Integer read GetResultDataLength;
    property resultIndicatorCount: Integer read GetResultIndicatorCount;
    property errorMessage: TcaoErrorMessage read FcaoErrorMessage write FcaoErrorMessage;
    property databaseConnection: TcaoDatabaseConnection read FcaoDatabaseConnection write FcaoDatabaseConnection;
    property databaseDataFormat: TcaoDatabaseDataFormat read FcaoDatabaseDataFormat write FcaoDatabaseDataFormat;
    property OleObject: cwbDatabaseCatalog read FcwbDatabaseCatalog;
  end;

implementation

{
Create OLE object and initilize properties.
}

constructor TcaoDatabaseCatalog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FcwbDatabaseCatalog := CocwbDatabaseCatalog.Create;
  FSetupCalled := false;
  FdataRetrievalMode := caodbSynchronousRetrieval;
  FdataToRetrieve := [caodbRetrieveData,caodbRetrieveDataFormat];
  FdefaultFileAttributes := caodbAllFilesAttributes;
  FdefaultFileInfoOrdering := caodbDefaultCatalogOrdering;
  FdefaultNullableIndicator := caodbNotNullable;
  FdefaultStatementType := caodbAllStatements
end;

{
Return formatted result code text.
}

function TcaoDatabaseCatalog.ResultCodeText: String;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
Return error mesage or formatted result code text.
}

function TcaoDatabaseCatalog.GetErrMsg: String;
var
  S: string;
  errorClass: Smallint;
  errorCode: Integer;
  messageID: WideString;
  message1: WideString;
  message2: WideString;
  CauseStr: string;
  RecoveryStr: string;
  RecoveryPos: integer;
begin
  S := FcaoErrorMessage.GetErrMsg;
  if S <> '' then
  begin
    GetHostErrInfo(errorClass,errorCode,messageID,message1,message2);
    RecoveryPos := Pos('Recovery',message2);
    if RecoveryPos > 0 then
    begin
      CauseStr := Copy(message2,1,RecoveryPos-1);
      RecoveryStr := Copy(message2,RecoveryPos,Length(message2)-RecoveryPos)
    end
    else
    begin
      CauseStr := message2;
      RecoveryStr := '';
    end;
    S := Format('%s class: %d, code: %d, message ID: %s, %s'#10#10'%s'#10#10'%s',
    [S,errorClass,errorCode,messageID,message1,CauseStr,RecoveryStr])
  end
  else
    S := ResultCodeText;
  Result := S
end;

{
This property specifies when the result data set should be retrieved from the
AS/400.  Possible values are: cwbdbSynchronousRetrieval and
cwbdbAsynchronousRetrieval.  If the value is cwbdbSynchronousRetrieval, the
result data will be retrieved immediately, when a request is made for it.  If
the value is cwbdbAsynchronousRetrieval, the result data will not be retrieved
until an attempt is made to access it, by calling one of the cwbdbGetResult*
methods or by retrieving one of the result* properties.  When an attempt is
made to access one part of the result data, all result data for that request
will be retrieved.  If this property is not set, cwbdbSynchronousRetrieval is
used.  This is a read/write property.

Get OLE property dataRetrievalMode if setup was called.  Otherwise, get private
variable.
}

function TcaoDatabaseCatalog.GetDataRetrievalMode: TcaoCatDataRetrievalMode;
begin
  if FSetupCalled then
    case FcwbDatabaseCatalog.dataRetrievalMode of
      cwbdbSynchronousRetrieval:
        Result := caodbSynchronousRetrieval;
      cwbdbAsynchronousRetrieval:
        Result := caodbAsynchronousRetrieval
      else
        Result := caodbSynchronousRetrieval
    end
  else
    Result := FdataRetrievalMode
end;

{
Set OLE property dataRetrievalMode if setup was called. Otherwise, set private
variable.
}

procedure TcaoDatabaseCatalog.SetDataRetrievalMode(RetMode: TcaoCatDataRetrievalMode);
begin
  if FSetupCalled then
    case RetMode of
      caodbSynchronousRetrieval:
        FcwbDatabaseCatalog.dataRetrievalMode := cwbdbSynchronousRetrieval;
      caodbAsynchronousRetrieval:
        FcwbDatabaseCatalog.dataRetrievalMode := cwbdbAsynchronousRetrieval
    end
  else
    FdataRetrievalMode := RetMode
end;

{
This property specifies which part(s) of the result set should be retrieved
from the AS/400.  This may any combination of the following bit constants ORed
together.

	For a cwbDatabaseCatalog object:

cwbdbRetrieveData - data and indicators
cwbdbRetrieveDataFormat - data format

	For a cwbDatabaseSQL object:

cwbdbRetrieveData - data and indicators
cwbdbRetrieveDataFormat - data format
cwbdbRetrieveSQLCA - SQLCA information
cwbdbRetrieveParmMarkerFormat - parameter marker format

If this property is not set, the entire result set is retrieved.  If a certain
type of data is specifed but, it is not part of the result set for a particular
request, no attempt is made to retrieve it.  This is a read/write property.

Get OLE property dataToRetrieve if setup was called.  Otherwise, get private
variable.
}

function TcaoDatabaseCatalog.GetDataToRetrieve: TcaoCatDataToRetrieveSet;
var
  DataRet: SmallInt;
begin
  if FSetupCalled then
  begin
    DataRet := FcwbDatabaseCatalog.dataToRetrieve;
    Result := [];
    if (DataRet and Smallint(cwbdbRetrieveData)) =
    SmallInt(cwbdbRetrieveData) then
      Result := Result+[caodbRetrieveData];
    if (DataRet and Smallint(cwbdbRetrieveDataFormat)) =
    SmallInt(cwbdbRetrieveDataFormat) then
      Result := Result+[caodbRetrieveDataFormat]
  end
  else
    Result := FdataToRetrieve
end;

{
Set OLE property dataRetrievalMode if setup was called. Otherwise, set private
variable.
}

procedure TcaoDatabaseCatalog.SetDataToRetrieve(DataRet: TcaoCatDataToRetrieveSet);
var
  Value: SmallInt;
begin
  if FSetupCalled then
  begin
    Value := 0;
    if caodbRetrieveData in DataRet then
      Value := Value or ShortInt(cwbdbRetrieveData);
    if caodbRetrieveDataFormat in DataRet then
      Value := Value or ShortInt(cwbdbRetrieveDataFormat);
    FcwbDatabaseCatalog.dataToRetrieve := Value
  end
  else
    FdataToRetrieve := DataRet
end;

{
This property specifies which types of files should be queried for a
cwbdbRetrieveFileInfo request.  Possible values are:

cwbdbAllFilesAttributes
cwbdbPhysicalFilesAttributes,
cwbdbLogicalFilesAttributes
cwbdbODBCTablesAttributes
cwbdbODBCViewsAttributes.

If this property is not set, cwbdbAllFilesAttributes is used.  This is a
read/write property.

Get OLE property defaultFileAttributes if setup was called.  Otherwise, get
private variable.
}

function TcaoDatabaseCatalog.GetDefaultFileAttributes: TcaoDefaultFileAttributes;
begin
  if FSetupCalled then
    case FcwbDatabaseCatalog.defaultFileAttributes of
      cwbdbAllFilesAttributes:
        Result := caodbAllFilesAttributes;
      cwbdbPhysicalFilesAttributes:
        Result := caodbPhysicalFilesAttributes;
      cwbdbLogicalFilesAttributes:
        Result := caodbLogicalFilesAttributes;
      cwbdbODBCTablesAttributes:
        Result := caodbODBCTablesAttributes;
      cwbdbODBCViewsAttributes:
        Result := caodbODBCViewsAttributes
      else
        Result := caodbAllFilesAttributes
    end
  else
    Result := FdefaultFileAttributes
end;

{
Set OLE property defaultFileAttributes if setup was called. Otherwise, set
private variable.
}

procedure TcaoDatabaseCatalog.SetDefaultFileAttributes(FileAtt: TcaoDefaultFileAttributes);
begin
  if FSetupCalled then
    case FileAtt of
      caodbAllFilesAttributes:
        FcwbDatabaseCatalog.defaultFileAttributes := cwbdbAllFilesAttributes;
      caodbPhysicalFilesAttributes:
        FcwbDatabaseCatalog.defaultFileAttributes := cwbdbPhysicalFilesAttributes;
      caodbLogicalFilesAttributes:
        FcwbDatabaseCatalog.defaultFileAttributes := cwbdbLogicalFilesAttributes;
      caodbODBCTablesAttributes:
        FcwbDatabaseCatalog.defaultFileAttributes := cwbdbODBCTablesAttributes;
      caodbODBCViewsAttributes:
        FcwbDatabaseCatalog.defaultFileAttributes := cwbdbODBCViewsAttributes;
    end
  else
    FdefaultFileAttributes := FileAtt
end;

{
This property specifies the order in which data should be returned on a
cwbdbRetrieveFileInfo method call.  Possible values are:

cwbdbDefaultCatalogOrdering
cwbdbODBCTableOrdering
cwbdbODBCTablePrivilegeOrder

If this property is not set, cwbdbDefaultCatalogOrdering is used.  This is a
read/write property.

Get OLE property defaultFileInfoOrdering if setup was called.  Otherwise, get
private variable.
}

function TcaoDatabaseCatalog.GetDefaultFileInfoOrdering: TcaoDefaultFileInfoOrdering;
begin
  if FSetupCalled then
    case FcwbDatabaseCatalog.defaultFileInfoOrdering of
      cwbdbDefaultCatalogOrdering:
        Result := caodbDefaultCatalogOrdering;
      cwbdbODBCTableOrdering:
        Result := caodbODBCTableOrdering;
      cwbdbODBCTablePrivilegeOrder:
        Result := caodbODBCTablePrivilegeOrder
      else
        Result := caodbDefaultCatalogOrdering
    end
  else
    Result := FdefaultFileInfoOrdering
end;

{
Set OLE property defaultFileInfoOrdering if setup was called. Otherwise, set
private variable.
}

procedure TcaoDatabaseCatalog.SetDefaultFileInfoOrdering(FileOrd: TcaoDefaultFileInfoOrdering);
begin
  if FSetupCalled then
    case FileOrd of
      caodbDefaultCatalogOrdering:
        FcwbDatabaseCatalog.defaultFileInfoOrdering := cwbdbDefaultCatalogOrdering;
      caodbODBCTableOrdering:
        FcwbDatabaseCatalog.defaultFileInfoOrdering := cwbdbODBCTableOrdering;
      caodbODBCTablePrivilegeOrder:
        FcwbDatabaseCatalog.defaultFileInfoOrdering := cwbdbODBCTablePrivilegeOrder
    end
  else
    FdefaultFileInfoOrdering := FileOrd
end;

{
This property indicates whether to return special columns, which can have a
null value, on a cwbdbRetrieveSpecialColumnInfo call.  Possible values are:
cwbdbNotNullable (do not return columns that can have null values) and
cwbdbNullable (return columns that can have null values).  If this property is
not set, cwbdbNotNullable is used. This is a read/write property.

Get OLE property defaultNullableIndicator if setup was called.  Otherwise, get
private variable.
}

function TcaoDatabaseCatalog.GetDefaultNullableIndicator: TcaoDefaultNullableIndicator;
begin
  if FSetupCalled then
    case FcwbDatabaseCatalog.defaultNullableIndicator of
      cwbdbNotNullable:
        Result := caodbNotNullable;
      cwbdbNullable:
        Result := caodbNullable
      else
        Result := caodbNotNullable
    end
  else
    Result := FdefaultNullableIndicator
end;

{
Set OLE property defaultNullableIndicator if setup was called. Otherwise, set
private variable.
}

procedure TcaoDatabaseCatalog.SetDefaultNullableIndicator(NullInd: TcaoDefaultNullableIndicator);
begin
  if FSetupCalled then
    case NullInd of
      caodbNotNullable:
        FcwbDatabaseCatalog.defaultNullableIndicator := cwbdbNotNullable;
      caodbNullable:
        FcwbDatabaseCatalog.defaultNullableIndicator := cwbdbNullable
    end
  else
    FdefaultNullableIndicator := NullInd
end;

{
This property indicates the type of SQL statements for which information should
be retrieved when making cwbdbRetrievePackageStmtInfo method calls.  Possible
values are:

cwbdbAllStatements
cwbdbDeclareStatements
cwbdbSelectStatements
cwbdbExecStatements.

If this property is not set, cwbdbAllStatements is used.  This is a read/write
property.

Get OLE property defaultStatementType if setup was called.  Otherwise, get
private variable.
}

function TcaoDatabaseCatalog.GetDefaultStatementType: TcaoDefaultStatementType;
begin
  if FSetupCalled then
    case FcwbDatabaseCatalog.defaultStatementType of
      cwbdbAllStatements:
        Result := caodbAllStatements;
      cwbdbDeclareStatements:
        Result := caodbDeclareStatements;
      cwbdbSelectStatements:
        Result := caodbSelectStatements;
      cwbdbExecStatements:
        Result := caodbExecStatements
      else
        Result := caodbAllStatements
    end
  else
    Result := FdefaultStatementType
end;

{
Set OLE property defaultStatementType if setup was called. Otherwise, set
private variable.
}

procedure TcaoDatabaseCatalog.SetDefaultStatementType(StmtType: TcaoDefaultStatementType);
begin
  if FSetupCalled then
    case StmtType of
      caodbAllStatements:
        FcwbDatabaseCatalog.defaultStatementType := cwbdbAllStatements;
      caodbDeclareStatements:
        FcwbDatabaseCatalog.defaultStatementType := cwbdbDeclareStatements;
      caodbSelectStatements:
        FcwbDatabaseCatalog.defaultStatementType := cwbdbSelectStatements;
      caodbExecStatements:
        FcwbDatabaseCatalog.defaultStatementType := cwbdbExecStatements
    end
  else
    FdefaultStatementType := StmtType
end;

{
This property contains the size of the data portion of the result set, for the
most recent request. This property can be retrieved to to determine the size
of the data buffer needed for a cwbdbGetResultData call.  This is a read-only
property.
}

function TcaoDatabaseCatalog.GetResultDataLength: Integer;
begin
  Result := FcwbDatabaseCatalog.resultDataLength
end;

{
This property contains the number of the indicators for the result set of the
most recent request.  This property can be retrieved to to determine the size
of the indicator buffer needed for a cwbdbGetResultData call.  This is a
readonly property.
}

function TcaoDatabaseCatalog.GetResultIndicatorCount: Integer;
begin
  Result := FcwbDatabaseCatalog.resultIndicatorCount
end;

{
Handle caoErrorMessage, caoDatabaseConnection or caoDatabaseDataFormat
component being removed.
}

procedure TcaoDatabaseCatalog.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FcaoErrorMessage then
      FcaoErrorMessage := nil
    else
      if AComponent = FcaoDatabaseConnection then
        FcaoDatabaseConnection := nil
      else
        if AComponent = FcaoDatabaseDataFormat then
          FcaoDatabaseDataFormat := nil
  end
end;

{
This method initializes the object.  It must be called before any other method
or before accessing any of the objects properties.
}

procedure TcaoDatabaseCatalog.Setup;
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbSetup(
  FcaoDatabaseConnection.OleObject,FcaoDatabaseDataFormat.OleObject,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(ResultCodeText);
  FSetupCalled := true;
  // Set OLE properties from variables
  SetDataRetrievalMode(FdataRetrievalMode);
  SetDataToRetrieve(FdataToRetrieve);
  SetDefaultFileAttributes(FdefaultFileAttributes);
  SetDefaultFileInfoOrdering(FdefaultFileInfoOrdering);
  SetDefaultNullableIndicator(FdefaultNullableIndicator);
  SetDefaultStatementType(FdefaultStatementType)
end;

{
This method returns the data portion of the result set for the most recent
requesst.  If the dataToRetrieve property value does not include
cwbdbRetrieveData, no data will be returned.

Parameters:

dataBuf - Byte Array Variant

Specifies a buffer to be used to store the data.  (This buffer should be passed
as a SafeArray of unsigned characters.)

dataLen - Pointer to Long

Used to store the number of bytes stored in dataBuf, on output.  If the Return
Code is cwbBufferOverflow, this parameter will contain the size of the buffer
needed to satisfy this request.  The resultDataLength may also be retrieved,
before calling this method, to determine the buffer size.

indicatorBuf - Short Integer Array Variant

Specifies a buffer to be used to store the null data indicators.  Null
indicators are only returned if at least one of the columns in the result set
is null capable.  There is one indicator returned for each column of each row
in the result set.  Each entry specifes if the corresponding data value is null
(-1) or if it experienced a conversion error ('E' or 0x0045).  Otherwise, the
value is zero.  The indicators for the first row are stored in the array first,
followed by the indicators for the second row, and so on.  If the indicators
are not wanted, this value should be NULL.  (This should be passed as a
SafeArray of signed short Integers.)

indicatorCount - Pointer to Long

Used to store the number of null indicators stored in indicatorBuf, on output.
If the Return Code is cwbBufferOverflow, this parameter will contain the number
of indicators in the result set.  The resultIndicatorCount property value may
also be retrieved, before calling this method, to determine the number of
indicators.
}

procedure TcaoDatabaseCatalog.GetResultData(var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbGetResultData(
  dataBuf,dataLen,indicatorBuf,indicatorCount,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method puts the data format information of the result set, for the most
recent request, into the cwbDatabaseDataFormat object specified on the
cwbdbSetup method call.  After this method is called, the methods of the
cwbDatabaseDataFormat object can be used to access the data format information.
If the dataToRetrieve property value does not include cwbdbRetrieveDataFormat,
this method does nothing.
}

procedure TcaoDatabaseCatalog.GetResultDataFormat;
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbGetResultDataFormat(
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about a library or a list of libraries.

Parameters:

libraryName - String

Identifes the library(ies) for which information should be returned.  A value
of "*" means information should be returned for all libraries in the user's
library list.

retrieveInformation - Long

Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetLibraryName - name
cwbdbGetLibraryDesc - description
cwbdbGetLibraryOwner - owner

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveLibraryInfo(const libraryName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveLibraryInfo(
  libraryName,retrieveInformation,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method returns any host error information associated with the most recent
request.

Parameters:
errorClass - Pointer to Short Integer
Specifies a buffer where the error class will be stored.  This class indicates which database server module encountered an error.  Possible values are:

0 - no error
1 - SQL functional error
2 - SQL parameter error
3 - List functional error
4 - List parameter error
5 - DB2 functional error
6 - DB2 parameter error
7 - General server error
8 - User exit error

errorCode - Pointer to Long
Specifies a buffer where the return code from the server module will be stored.
messageID - Pointer to String
Provides a location to store the host message identifier.  (This should be
passed as a pointer to a BSTR.)
message1 - Pointer to String
Provides a location to store the first host message.  (This should be passed as
a pointer to a BSTR.)
message2 - Pointer to String
Provides a location to store the second host message identifier.  (This should
be passed as a pointer to a BSTR.)
}

procedure TcaoDatabaseCatalog.GetHostErrInfo(var errorClass: Smallint; var errorCode: Integer; var messageID: WideString; var message1: WideString; var message2: WideString);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbGetHostErrInfo(errorClass,errorCode,
  messageID,message1,message2,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about the fields in an AS/400 file.

Parameters:
libraryName - String
Identifies the library in which the file exists.
fileName - String
Identifies the file to be queried.
fieldName - String
Identifes the field(s) for which information should be returned.  A value of
"*" means information should be returned for all fields.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetFldLib - library name
cwbdbGetFldRemarks - remarks
cwbdbGetFldFile - file name
cwbdbGetFldName - field name
cwbdbGetFldDesc - description
cwbdbGetFldDataType - data type
cwbdbGetFldLen - length
cwbdbGetFldNull - null capable
cwbdbGetFldRadix - radix
cwbdbGetFldPrec - precision
cwbdbGetFldScale - scale

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveFieldInfo(const libraryName, fileName, fieldName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveFieldInfo(
  libraryName,fileName,fieldName,retrieveInformation,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about files on an AS/400.

Parameters:
libraryName - String
Identifies the library in which the file exists.
fileName - String
Identifes the file(s) for which information should be returned.  A value of "*"
means information should be returned for all files.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetFileLib - library name
cwbdbGetFileRemarks - remarks
cwbdbGetFileName - file name
cwbdbGetFileAttrib - attributes
cwbdbGetFileDesc - description
cwbdbGetFileColCnt - column count
cwbdbGetFileAuth - current user authority

Before this method is called, the following property(ies) may be set:

defaultFileAttributes
defaultFileInfoOrdering

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveFileInfo(const libraryName, fileName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveFileInfo(
  libraryName,fileName,retrieveInformation,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about foreign keys for an AS/400 file.

Parameters:
foreignKeyLibraryName - String
Identifiesthe library in which the table containing the dependent key exits.
foreignKeyFileName - String
Identifies the name of the table containing the dependent key.
primaryKeyLibraryName - String
Identifies the library in which the table containing the parent key exists.
primaryKeyFileName - String
Identifies the name of the table containing the parent key.

retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetPrKeyLib - primary key library name
cwbdbGetPrKeyFile - primary key file name
cwbdbGetPrKeyColID - primary key column identifier
cwbdbGetPrKeyColSeq - primary key column sequence
cwbdbGetFgPrkeyLib - foreign key primary key library name
cwbdbGetFgPrkeyFile - foreign key primary key column identifier
cwbdbGetFgPrkeyLib - foreign key primary key library name
cwbdbGetFgPrkeyFile - foreign key primary key file name
cwbdbGetFgPrkeyColID - foreign key primary column idenfitier

cwbdbGetFgKeyLib - foreign key library name
cwbdbGetFgKeyFile - foreign key file name
cwbdbGetFgKeyColID - foreign key column identifer
cwbdbGetFgKeySeq - foreign key sequence
cwbdbGetFgKeyUpdate - foreign key update rule
cwbdbGetFgKeyDelete - foreign key delete rule
cwbdbGetFgKeyUpdate - foreign key update rule
cwbdbGetFgKeyDelete - foreign key delete rule

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveForeignKeyInfo(const foreignKeyLibraryName, foreignKeyFileName, primaryKeyLibraryName, primaryKeyFileName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveForeignKeyInfo(
  foreignKeyLibraryName,foreignKeyFileName,primaryKeyLibraryName,
  primaryKeyFileName,retrieveInformation,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about the indexes for an AS/400 file.
Parameters:
libraryName - String
Identifies the library containing the table for which the indexes are defined.
fileName - String
Identifies the table for which the indexes are defined.
indexType - Short Integer
Identifies the index rule to follow.  Possible values are:

cwbdbUniqueIndex - index is unique
cwbdbDuplicateIndex - duplicates are allowed
cwbdbDupNullIndex - duplicate null values are allowed

retrieveInformation - Long
Identifies the type of information to be retrieved.  This may be any
combination of the following bit constants ORed together:
cwbdbGetIdxTblLib - library name
cwbdbGetIdxTblName - table name
cwbdbGetIdxUnique - index unique rule
cwbdbGetIdxIdxLib - index library name
cwbdbGetIdxIdxName - index name
cwbdbGetIdxColCnt - column count
cwbdbGetIdxColName - column name
cwbdbGetIdxColSeq - column sequence
cwbdbGetIdxCollat - collation (ordering)

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveIndexInfo(const libraryName, fileName: WideString; indexType: Smallint; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveIndexInfo(
  libraryName,fileName,indexType,retrieveInformation,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about members of an AS/400 file.

Parameters:
libraryName - String
Identifies the library in which the file exists.
fileName - String
Identifies the file to be queried.
memberName - String
Identifes the member(s) for which information should be returned.  A value of
"*" means information should be returned for all members.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetMbrLib - library name
cwbdbGetMbrFile - file name
cwbdbGetMbrName - member name
cwbdbGetMbrDesc- description

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveMemberInfo(const libraryName, fileName, memberName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveMemberInfo(
  libraryName,fileName,memberName,retrieveInformation,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about statements stored in a SQL package on
the AS/400.

Parameters:
libraryName - String
Identifies the library which contains the package.
packageName - String
Identifies the package to be queried.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetSQLStmtLib - library name
cwbdbGetSQLStmtPkg - package name
cwbdbGetSQLStmtName - statement name
cwbdbGetSQLStmtType - statement type
cwbdbGetSQLStmtText - statement text
cwbdbGetSQLStmtPmCnt - parameter count

Before this method is called, the following property(ies) may be set:

defaultStatementType

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrievePackageStmtInfo(const libraryName, packageName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrievePackageStmtInfo(
  libraryName,packageName,retrieveInformation,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about primary keys for an AS/400 file.

Parameters:
primaryKeyLibraryName - String
Identifies the library containing the table for which the primary key has been
defined.
primaryKeyFileName - String
Identifies the table for which the primary key has been defined.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetPrKeyLib - library name
cwbdbGetPrKeyFile - file name
cwbdbGetPrKeyColID - column identifier
cwbdbGetPrKeyColSeq - column sequence

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrievePrimaryKeyInfo(const primaryKeyLibraryName, primaryKeyFileName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrievePrimaryKeyInfo(
  primaryKeyLibraryName,primaryKeyFileName,retrieveInformation,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about relational databases on the AS/400.

Parameters:
RDBName - String
Identifes the relational database for which information is being requested.  A
value of * means information should be returned for all relational databases.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetRDBName - name
cwbdbGetRDBDevice - device name
cwbdbGetRDBMode - mode name
cwbdbGetRDBRmtloc - remote location name
cwbdbGetRDBLocloc - local location name
cwbdbGetRDBRmtnet - remote network identifier
cwbdbGetRDBTpname - transaction program name
cwbdbGetRDBDesc - description
cwbdbGetRDBTpndisp -TPN display name
cwbdbGetRDBPgm - program name
cwbdbGetRDBPgmlib - program library name
cwbdbGetRDBPgmlevel - program level

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveRDBInfo(const RDBName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveRDBInfo(
  RDBName,retrieveInformation,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about the record formats for an AS/400 file.

Parameters:
libraryName - String
Identifies the library in which the file exists.
fileName - String
Identifies the file to be queried.
formatName - String
Identifes the format(s) for which information should be returned.  A value of
"*" means information should be returned for all formats.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetFmtLib - library name
cwbdbGetFmtFile - file name
cwbdbGetFmtName - format name
cwbdbGetFmtRecLen - record length
cwbdbGetFmtDesc- description

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveRecordFormatInfo(const libraryName, fileName, formatName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveRecordFormatInfo(
  libraryName,fileName,formatName,retrieveInformation,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about the optimal set of columns that uniquely
identifies a row in the table.

Parameters:
libraryName - String
Identifies the library in which the file exists.
fileName - String
Identifies the file to be queried.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetSpColLib - library name
cwbdbGetSpColTable - table name
cwbdbGetSpColColName - column name
cwbdbGetSpColDataType - data type
cwbdbGetSpColPrecision - column precision
cwbdbGetSpColLength - column length
cwbdbGetSpColScale - column scale

Before this method is called, the following property(ies) may be set:

defaultNullableIndicator

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveSpecialColumnInfo(const libraryName, fileName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveSpecialColumnInfo(
  libraryName,fileName,retrieveInformation,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method retrieves information about a SQL package on the AS/400.

Parameters:
libraryName - String
Identifies the library which contains the package.
packageName - String
Identifies the package to be queried.
retrieveInformation - Long
Identifies the type of information to be retrieved.  This may any combination
of the following bit constants ORed together:

cwbdbGetSQLPkgLib - library name
cwbdbGetSQLPkgName - package name
cwbdbGetSQLPkgDesc- description

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseCatalog.RetrieveSQLPackageInfo(const libraryName, packageName: WideString; retrieveInformation: Integer);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbRetrieveSQLPackageInfo(
  libraryName,packageName,retrieveInformation,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

{
This method provides a way to switch which cwbDatabaseConnection object is used
for the catalog requests made by this object.

Parameters:
connection - Object
Identifies the new cwbDatabaseConnection object to be used for servicing
requests of this object.
}

procedure TcaoDatabaseCatalog.ChangeConnection(connection: IDispatch);
begin
  FReturnCode := FcwbDatabaseCatalog.cwbdbChangeConnection(
  connection,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseCatalog.Create(GetErrMsg)
end;

end.
